@@ -43,6 +43,8 @@
 #include "panel-stock-icons.h"
 #include "panel-typebuiltins.h"
 
+G_DEFINE_TYPE (PanelMenuBar, panel_menu_bar, GTK_TYPE_MENU_BAR);
+
 #define PANEL_MENU_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BAR, PanelMenuBarPrivate))
 
 struct _PanelMenuBarPrivate {
@@ -62,7 +64,7 @@ enum {
 	PROP_ORIENTATION,
 };
 
-static GObjectClass *parent_class;
+static void panel_menu_bar_update_text_gravity (PanelMenuBar *menubar);
 
 static gboolean
 panel_menu_bar_enter_applications (GtkWidget        *widget,
@@ -169,8 +171,7 @@ panel_menu_bar_setup_tooltip (PanelMenuBar *menubar)
 }
 
 static void
-panel_menu_bar_instance_init (PanelMenuBar      *menubar,
-			      PanelMenuBarClass *klass)
+panel_menu_bar_init (PanelMenuBar *menubar)
 {
 	GtkWidget *image;
 
@@ -200,6 +201,11 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 			       menubar->priv->desktop_item);
 
 	panel_menu_bar_setup_tooltip (menubar);
+
+	panel_menu_bar_update_text_gravity (menubar);
+	g_signal_connect (menubar, "screen-changed",
+			  G_CALLBACK (panel_menu_bar_update_text_gravity),
+			  NULL);
 }
 
 static void
@@ -280,7 +286,7 @@ panel_menu_bar_size_allocate (GtkWidget     *widget,
 	old_allocation.width  = widget->allocation.width;
 	old_allocation.height = widget->allocation.height;
 
-	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+	GTK_WIDGET_CLASS (panel_menu_bar_parent_class)->size_allocate (widget, allocation);
 
 	if (old_allocation.x      == allocation->x &&
 	    old_allocation.y      == allocation->y &&
@@ -303,8 +309,6 @@ panel_menu_bar_class_init (PanelMenuBarClass *klass)
 	GObjectClass   *gobject_class = (GObjectClass   *) klass;
 	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class->get_property = panel_menu_bar_get_property;
         gobject_class->set_property = panel_menu_bar_set_property;
 
@@ -332,32 +336,6 @@ panel_menu_bar_class_init (PanelMenuBarClass *klass)
 		"class \"PanelMenuBar\" style \"panel-menubar-style\"");
 }
 
-GType
-panel_menu_bar_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (PanelMenuBarClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) panel_menu_bar_class_init,
-			NULL,
-			NULL,
-			sizeof (PanelMenuBar),
-			0,
-			(GInstanceInitFunc) panel_menu_bar_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (
-				GTK_TYPE_MENU_BAR, "PanelMenuBar", &info, 0);
-	}
-
-	return type;
-}
-
 static gboolean
 panel_menu_bar_on_expose (GtkWidget      *widget,
 			  GdkEventExpose *event,
@@ -523,6 +501,27 @@ panel_menu_bar_change_background (PanelMenuBar *menubar)
 						      GTK_WIDGET (menubar));
 }
 
+static void
+set_item_text_gravity (GtkWidget *item)
+{
+	GtkWidget    *label;
+	PangoLayout  *layout;
+	PangoContext *context;
+
+	label = GTK_BIN (item)->child;
+	layout = gtk_label_get_layout (GTK_LABEL (label));
+	context = pango_layout_get_context (layout);
+	pango_context_set_base_gravity (context, PANGO_GRAVITY_AUTO);
+}
+
+static void
+panel_menu_bar_update_text_gravity (PanelMenuBar *menubar)
+{
+	set_item_text_gravity (menubar->priv->applications_item);
+	set_item_text_gravity (menubar->priv->places_item);
+	set_item_text_gravity (menubar->priv->desktop_item);
+}
+
 static void
 set_item_text_angle_and_alignment (GtkWidget *item,
 				   double     text_angle,
