@@ -341,10 +341,10 @@ panel_error_dialog (GtkWindow  *parent,
 	}
 
 	dialog = gtk_message_dialog_new (parent, 0, GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_OK, primary_text);
+					 GTK_BUTTONS_OK, "%s", primary_text);
 	if (secondary_text != NULL)
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  secondary_text);
+							  "%s", secondary_text);
 
 	gtk_widget_add_events (dialog, GDK_KEY_PRESS_MASK);
 	g_signal_connect (dialog, "event",
@@ -745,10 +745,16 @@ panel_lock_screen_action_get_command (const char *action)
 		}
 	} else if (strcmp (action, "activate") == 0
 		   || strcmp (action, "lock") == 0) {
-		if (use_gscreensaver) {
-			command = g_strdup_printf ("gnome-screensaver-command --%s", action);
+		/* Neither gnome-screensaver or xscreensaver allow root
+		 * to lock the screen */
+		if (geteuid () == 0) {
+			command = NULL;
 		} else {
-			command = g_strdup_printf ("xscreensaver-command -%s", action);
+			if (use_gscreensaver) {
+				command = g_strdup_printf ("gnome-screensaver-command --%s", action);
+			} else {
+				command = g_strdup_printf ("xscreensaver-command -%s", action);
+			}
 		}
 	}
 
